<?php

namespace Ls\Replication\Ui\DataProvider;

use Exception;
use \Ls\Core\Model\LSR;
use \Ls\Replication\Helper\ReplicationHelper;
use Magento\Framework\Api\Filter;
use Magento\Framework\App\Request\Http;
use Magento\Framework\Module\Dir\Reader;
use Magento\Framework\View\Element\UiComponent\DataProvider\DataProviderInterface;
use Magento\Framework\Xml\Parser;
use Magento\Ui\DataProvider\AbstractDataProvider;
use Magento\Ui\DataProvider\AddFieldToCollectionInterface;
use Magento\Ui\DataProvider\AddFilterToCollectionInterface;

/**
 * Class ProductDataProvider
 */
class CronsProvider extends AbstractDataProvider implements DataProviderInterface
{

    /**
     * @var AddFieldToCollectionInterface[]
     */
    public $addFieldStrategies;

    /**
     * @var AddFilterToCollectionInterface[]
     */
    public $addFilterStrategies;

    /**
     * @var Http
     */
    public $request;
    /**
     * @var Reader
     */
    public $moduleDirReader;
    /** @var LSR */
    public $lsr;
    /**
     * @var Parser
     */
    private $parser;

    /** @var ReplicationHelper */
    public $rep_helper;

    /**
     * CronsProvider constructor.
     * @param string $name
     * @param string $primaryFieldName
     * @param string $requestFieldName
     * @param Http $request
     * @param Reader $moduleDirReader
     * @param Parser $parser
     * @param LSR $LSR
     * @param ReplicationHelper $repHelper
     * @param array $meta
     * @param array $data
     */
    public function __construct(
        $name,
        $primaryFieldName,
        $requestFieldName,
        Http $request,
        Reader $moduleDirReader,
        Parser $parser,
        LSR $LSR,
        ReplicationHelper $repHelper,
        array $meta = [],
        array $data = []
    ) {
        parent::__construct($name, $primaryFieldName, $requestFieldName, $meta, $data);
        $this->request         = $request;
        $this->moduleDirReader = $moduleDirReader;
        $this->parser          = $parser;
        $this->lsr             = $LSR;
        $this->rep_helper      = $repHelper;
    }

    /**
     * @return array|mixed
     * @throws Exception
     */
    public function getData()
    {
        $cronsGroupListing = $this->readCronFile();
        $items             = [];
        $counter           = 1;
        $this->rep_helper->flushByTypeCode('config');
        foreach ($cronsGroupListing as &$cronlist) {
            $path = '';
            if (array_key_exists('_value', $cronlist['_value']['job'])) {
                $cronlist['_value']['job'] = [$cronlist['_value']['job']];
            }
            if ($cronlist['_attribute']['id'] == "replication" || $cronlist['_attribute']['id'] == "sync_operations") {
                $condition = __("Flat to Magento");
            } elseif ($cronlist['_attribute']['id'] == "flat_replication") {
                $condition = __("Omni to Flat");
                $path      = LSR::CRON_STATUS_PATH_PREFIX;
            } else {
                $condition = '';
            }
            foreach ($cronlist['_value']['job'] as $joblist) {
                $fullReplicationStatus = 0;
                $cronName              = $joblist['_attribute']['name'];
                if ($path != '') {
                    $pathNew               = $path . $cronName;
                    $fullReplicationStatus = $this->lsr->getStoreConfig($pathNew);
                }
                if ($cronName == 'repl_attributes') {
                    $cronAttributeCheck        = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_ATTRIBUTE);
                    $cronAttributeVariantCheck = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_ATTRIBUTE_VARIANT);
                    if ($cronAttributeCheck && $cronAttributeVariantCheck) {
                        $fullReplicationStatus = 1;
                    }
                }
                if ($cronName == 'repl_category') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_CATEGORY);
                }
                if ($cronName == 'repl_products') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_PRODUCT);
                }
                if ($cronName == 'repl_discount_create') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_DISCOUNT);
                }
                if ($cronName == 'repl_price_sync') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_PRODUCT_PRICE);
                }
                if ($cronName == 'repl_inventory_sync') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_PRODUCT_INVENTORY);
                }
                if ($cronName == 'repl_item_updates_sync') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_ITEM_UPDATES);
                }
                if ($cronName == 'repl_item_images_sync') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_ITEM_IMAGES);
                }
                if ($cronName == 'repl_attributes_value_sync') {
                    $fullReplicationStatus = $this->lsr->getStoreConfig(LSR::SC_SUCCESS_CRON_ATTRIBUTES_VALUE);
                }
                $lastExecute = $this->rep_helper->convertDateTimeIntoCurrentTimeZone(
                    $this->lsr->getStoreConfig('ls_mag/replication/last_execute_' . $cronName),
                    'd M, Y h:i:s A'
                );
                $statusStr   = ($fullReplicationStatus == 1) ?
                    '<div class="flag-green custom-grid-flag">Complete</div>' :
                    '<div class="flag-yellow custom-grid-flag">Pending</div>';
                if (strpos($cronName, '_reset') !== false || $cronName == "sync_version") {
                    $statusStr = '';
                }
                $items[] = [
                    'id'                    => $counter,
                    'fullreplicationstatus' => $statusStr,
                    'label'                 => $cronName,
                    'lastexecuted'          => $lastExecute,
                    'value'                 => $joblist['_attribute']['instance'],
                    'condition'             => $condition
                ];
                $counter++;
            }
        }
        // @codingStandardsIgnoreStart
        $pageSize    = (int)$this->request->getParam('paging')['pageSize'];
        $pageCurrent = (int)$this->request->getParam('paging')['current'];
        // @codingStandardsIgnoreEnd
        $pageOffset = ($pageCurrent - 1) * $pageSize;

        return [
            'totalRecords' => count($items),
            'items'        => array_slice($items, $pageOffset, $pageOffset + $pageSize)
        ];
    }

    /**
     * @return mixed
     */
    public function readCronFile()
    {
        try {
            $filePath    = $this->moduleDirReader->getModuleDir('etc', 'Ls_Replication') . '/crontab.xml';
            $parsedArray = $this->parser->load($filePath)->xmlToArray();
            return $parsedArray['config']['_value']['group'];
        } catch (Exception $e) {
        }
    }

    /**
     * @param int $offset
     * @param int $size
     */
    public function setLimit($offset, $size)
    {
    }

    /**
     * @param string $field
     * @param string $direction
     */
    public function addOrder($field, $direction)
    {
    }

    /**
     * @param Filter $filter
     * @return mixed|void
     */
    public function addFilter(Filter $filter)
    {
    }

    /**
     * @param SearchResultInterface $searchResult
     * @return array
     */
    public function searchResultToOutput(SearchResultInterface $searchResult)
    {
    }

    public function getItems()
    {
    }
}
